function [dots]= Riser128_model(t,x,u,flow1,flow2,flow3,WC_r,Kp_r,P_s) 


% the purpuse of this model is 1. to find the amount of oil and gas produced
% at the top of a riser before connecting it to a separator. 2. to find the manifold pressure at the bottom of the riser
 


import casadi.*

%%%%%%%%%%%%%%%%%%
%  the system is represented such that xdot= f(x,u) , y=h(x,u) , x
%  represents the mass of the phases liquid and gas which evolves according
%  to f. the function h defines the variables y which gathers the input
%  pressure and output mass flow rat variables for each phase
%  ODE 
x(1)      =   x(1)*1000;  % check why you have to multiple by 1000 
x(2)      =   x(2)*1000;

w_out     =   x(3); 
w_g_out   =   x(4); 
w_l_out   =   x(5);

gas_flow1 = flow1(1);         oil_flow1 = flow1(2);   % this is meant to be liquid flow and not oil flow 
gas_flow2 = flow2(1);         oil_flow2 = flow2(2); 
gas_flow3 = flow3(1);         oil_flow3 = flow3(2);   

g_f_in    = [gas_flow1, gas_flow2, gas_flow3];  
o_f_in    = [oil_flow1,oil_flow2,oil_flow3];
%%%%%%%%%%%%%%%%%%%%%%



% call in Parameters 
WC_r    = WC_r/100;   % mass water cut [fraction]
P_s     = 20*1e5;     % Pressure after riser production choke [Pa] this can be adjusted later
Kp_r    = Kp_r*1e-3;  % Production choke constant [-]

% given Parameters
Sr      = 0.0507;     % cross section riser [m2]
Lr      = 250;        % length riser [m]
Vr      = Sr*Lr;      % volume riser [m3]
rho_w   = 1000;       % water density [kg/m3]
rho_o   = 900;        % oil density [kg/m3]
rho_l   = 1/(WC_r/rho_w + (1-WC_r)/rho_o);        % liquid density [kg/m3] 
R       = 8.314;      % universal gas constant [J/mol/K]
g       = 9.81;       % gravity [m/s2]
Tr      = 369.4;      % riser temp. [K]
Mg      = 16.7e-3;    % gas molecular weight [kg/mol]
mu_o    = 3.64e-3;
mu      = mu_o/(1+WC_r)^2.5;    % viscosity [Pa.s]
Dr      = 0.134;      % riser diameter [m]



a_l_ave      = x(2)/Vr/rho_l;  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% first step is to find the manifold pressure 
rho_g_t      = x(1)/(Vr-x(2)/rho_l);
rho_mix_ave  = sum(x(1:2))/Vr;       % average density around the entire riser  
Q            = a_l_ave*w_out/rho_mix_ave;     % average volumetric flow 
P_r_t        = R*Tr*rho_g_t/Mg;       % pressure at the top of the riser 
Pd           = rho_mix_ave*g*Lr;      % pressure drop from t to b of riser 
P_fr         = 128*mu*Lr*Q/(pi*Dr^4); % pressure liquid friction with inner surface area or riser 
Pm           = P_r_t+Pd+P_fr;    % pressure at manifold, first objective of this model 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% second step is to find mass flow rates of liquid and qas 
% [SARRY] try using only average and see how results vary 
% the reason for that is each well will have different densities of gas and
% liquid as they enter the bottom of the riser
% the reason we could do that is because we have Pm and this allows us to
% find the density at the maniflod aka bottom 

% a_l_ave      = x(2)/Vr/rho_l;                  % average liquid volume fraction in riser 
rho_g_b      = Pm*Mg/R/Tr;                     % Density at the bottom of riser 
a_l_b        = sum(o_f_in)*rho_g_b/(sum(g_f_in)*rho_l + sum(o_f_in)*rho_g_b); % liquid volume fraction at the bottom
a_l_t        = 2*a_l_ave - a_l_b;              % this is a relation from Jahanshahi and Skogestad (2011) 
rho_mix_t    = a_l_t*rho_l+(1-a_l_t)*rho_g_t;  % this is the density mix at the top since it will vary from bottom to top
% f            = w_out - Kp_r*u*sqrt(rho_mix_t*max(P_r_t-P_s,0)); % residule for fsolve of w_out
%  f            = w_out - Kp_r*u*sqrt(rho_mix_t*(P_r_t-P_s));
f            = w_out - Kp_r*u*(rho_mix_t*(P_r_t-P_s))^(0.5);
a_m_g_t      = (1-a_l_t)*rho_g_t/((1-a_l_t)*rho_g_t+a_l_t*rho_l); % gas mass fraction at top of the riser
f1           = w_g_out - a_m_g_t*w_out;        % residule for fsolve of w_g_out
f2           = w_l_out - (1-a_m_g_t)*w_out;    % residule for fsolve of w_l_out

xdot1        = sum(g_f_in)-w_g_out;            % gas mass flow over riser  
xdot2        = sum(o_f_in)-w_l_out;            % liquid mass flow over riser 

%%%%%%%%%%%%%%%%%%%%%%%%%%

% the output is [dots] of size 10:
%[xdot1, xdot2, f , f1 ,f2 , Pm , mass_gass_out, mass_water_out,mass_oil_out,P_r_t]  
dots=[xdot1*1e-3;xdot2*1e-3;f;f1;f2;1e-5*Pm;w_l_out];%w_g_out;WC_r*w_l_out;(1-WC_r)*w_l_out;P_r_t*1e-5]; % if additional outputs are desired



