function [well]= well128_model(t,x,u,Pm,P_res,PI,Kpr,GOR,WC)
% The purpose of this model is to represent mathematically a simplified
% dynamical model for a well with a gas lift. 1. we find the gas mass flow
% into the annulus thus we require to find some pressure relations. 2. we
% find the pressure on the bottom of the tubing next to the injection point
% thus finding mass gas flow injected to the tube. 3. we find bottom hole
% pressure and hence the reservoir mass flow. 4. we find the density on top
% of the tubing tto find the mass flow out and use it in the riser model.


import casadi.*

% State variables of our model are x1 mass gas in annulus, x2 mass gas in
% tubing,x3 mass liquid in tubing 
% xdot1 = w_g_in  - w_g_inj 
% xdot2 = w_g_inj + w_g_res - w_g_out 
% xdot3 = w_l_res - w_l_out 

% ODE variables
x(1)      = x(1)*1000; % mass of gas in annulus [kg]
x(2)      = x(2)*1000; % mass of gas in tubing
x(3)      = x(3)*1000; % mass of liquid in tubing 

% DAE vriables
w_res     = x(4);   %[kg/s]
w_g_inj   = x(5);
w_out     = x(6); 




%%%%%%%%%%%%%%%%%%%%
% Call in parameters
PI    = PI*1e-6;
WC    = WC/100;
GOR   = GOR/100;
Kpr   = Kpr*1e-3;
Pm    = Pm*1e5;
P_res = P_res*1e5;

% Parameters
Mg    = 16.7e-3;
Va    = 64.34;
Vt    = 25.03;
Ta    = 348;
R     = 8.314;
P_gs  = 140e5;
g     = 9.81;
Lt    = 2048;
La    = 2048;
Dt    = 0.134;
T_bh  = 400;
Tt    = 369.4;
Lbh   = 75;
Sbh   = 0.0141;
Vbh   = Lbh*Sbh;
Dbh   = 0.134;
Kinj  = 1.40e-4;
Kgs   = 9.98e-5;
rho_w = 1000;
rho_o = 900;
rho_l = 1/(WC/rho_w + (1-WC)/rho_o);
mu_o  = 3.64e-3;
mu    = mu_o/(1+WC)^2.5;
%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%
% first step is to find gas mass flow into the annulus
rho_g_a   = x(1)/Va;
rho_g_in  = P_gs*Mg/R/Ta;
P_a_t     = rho_g_a*R*Ta/Mg;
% w_g_in    = Kgs*u(2)*sqrt(rho_g_in*max(P_gs-P_a_t,0));
% w_g_in    = Kgs*u(2)*sqrt(rho_g_in*(P_gs-P_a_t));
w_g_in    = Kgs*u(2)*(rho_g_in*(P_gs-P_a_t))^(0.5);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a_l_ave        = (x(3)-Vbh*rho_l)/Vt/rho_l;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second step is to find gas mass flow injected into tubing 
dP_a         = rho_g_a*g*La;
P_a_b        = P_a_t + dP_a;
rho_g_t_ave  = x(2)/(Vt+Vbh-x(3)/rho_l);
rho_mix_ave  = (x(3)+x(2)-Vbh*rho_l)/Vt;
P_t_t        = rho_g_t_ave*R*Tt/Mg; 
Pd_t         = rho_mix_ave*g*Lt;

Q            =a_l_ave*w_out/rho_mix_ave;    

% and then we can add GOR to 
P_t_fr       = 128*mu*Lt*Q/(pi*Dt^4);  
P_t_b        = P_t_fr + Pd_t + P_t_t;
rho_g_a_b    = P_a_b*Mg/R/Ta;
% f1           = w_g_inj-Kinj*sqrt(rho_g_a_b*max(P_a_b-P_t_b,0));  
% f1           = w_g_inj-Kinj*sqrt(rho_g_a_b*(P_a_b-P_t_b));
f1           = w_g_inj-Kinj*(rho_g_a_b*(P_a_b-P_t_b))^(0.5);
%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%
% now we want to find mass flow from reservoir and out of the whole system 
% so we find pressure bottom hole in order to calculate the reservoir mass flow 


a_g_b_m        = GOR/(GOR+1); % gas mass fraction at the bottom of tubing
dP_bh          = rho_l*g*Lbh; 
Q              = w_res/rho_l;    % should we use the average wres
P_bh_fr        = 128*mu*Lbh*Q/pi/Dbh^4;
P_bh           = P_t_b+dP_bh+P_bh_fr;

% f2             = w_res -PI*max(P_res-P_bh,0); % include an fsolve 
f2             = w_res -PI*(P_res-P_bh);
w_l_res        = (1-a_g_b_m)*w_res;
w_g_res        = a_g_b_m*w_res;




%%%%%%%%%%%%%%%%%%%%
% after finding the reservoir mass flow we are then able to calculate the
% actual volume fractions at the bottom of the tubing when reservoir mass
% flow is included and no longer at the very hiiiigggh pressures of the
% well. using the new volume fraction we will be able to calculate the
% actual mixture density at the top of the tubing so that we have a correct
% mass flow out of the gas lift. 
rho_g_t_b      = P_t_b*Mg/R/T_bh;
 
% we could calculate our way up 
% a_l_ave        = (x(3)-Vbh*rho_l)/Vt/rho_l;
a_l_b          = w_l_res*rho_g_t_b/(rho_g_t_b*w_l_res + rho_l*(w_g_res+w_g_inj));% liquid volume fraction at bottom of tubing 
a_l_t          = 2*a_l_ave - a_l_b ;                                             % liquid volume fraaction at top of the tubing Jahashahi Skogestad 2011
a_g_t_m        = (1-a_l_t)*rho_g_t_ave/(a_l_t*rho_l+(1-a_l_t)*rho_g_t_ave);      % gas mass fraction at top of tubing

rho_mix_t_t    = rho_l*a_l_t + rho_g_t_ave*(1-a_l_t);

% f3             = w_out -Kpr*u(1)*sqrt(rho_mix_t_t*max(P_t_t-Pm,0)); 
% f3             = w_out -Kpr*u(1)*sqrt(rho_mix_t_t*(P_t_t-Pm)); 
f3             = w_out -Kpr*u(1)*(rho_mix_t_t*(P_t_t-Pm))^(0.5); 
w_g_out        = a_g_t_m*w_out;
w_l_out        = (1-a_g_t_m)*w_out;
%%%%%%%%%%%%%%%%%%%%%%





%%%%%%%%%%%%%%%%%%%%%%
% formatting our state equations xdot(1,2,3)

xdot1 = w_g_in  - w_g_inj;
xdot2 = w_g_inj + w_g_res - w_g_out;
xdot3 = w_l_res - w_l_out;
%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%
% Defining outputs of our model 
xdot        = 1e-3*[xdot1;xdot2;xdot3];
residuals   = [f1;f2;f3];
outflow     = [w_g_out;w_l_out;w_g_in];
y           = [1e-5*P_a_t; 1e-5*P_a_b; 1e-5*P_t_t; 1e-5*P_t_b; 1e-5*P_bh];

well        = [xdot;residuals;outflow;y];
%%%%%%%%%%%%%%%%%%%
